( 
  maze.muf     by Gyroplast   05/21/2000   v1.0
 
  Simulates a maze with a randomly linked pool of rooms.
  Setup:
    1. Create a maze environment room. EX: '@dig Maze Environment'
    2. @dig and @desc as many maze rooms as you like, at least
       10 are recommended, and set their parent to the Maze Environment room.
    3. Create exit @actions on each maze room, and link them to this program.
    4. Set properties to specify program behaviour. 
 
  Props on Environment Room [REQUIRED]:
    _maze/reward:[int]        -  Specifies the amount of pennies a fur
                                 receives when getting out of the maze alive.
 
    _maze/targetroom:[dbref]  -  Set this to the room you want a fur to finally
                                 reach after getting through the maze.
   
    _maze/defaultchance:[int] -  Sets the chance of rooms to be chosen.
                                 The program randomly picks a room from the
                                 pool, then does another check with the
                                 'chance' setting. The value means percentage,
                                 so 0 will never choose any room, whilst 100
                                 will always accept the picked room. 
                                 If a room fails the test, another random
                                 room is picked until a check succeeds.
                                 Values >100 count as 100, 50 usually is a
                                 good choice. Superceded by _maze/chance.
  
    _maze/starthp:[int]       -  Amount of HitPoints everyfur starts with.
                                 Cannot be negative. Value depends on how
                                 deadly the traps are. A player is sent back
                                 home when his HP reach 0.
   
    _maze/minsteps:[int]      -  Set a minimum amount of rooms a fur must
                                 pass before an exitroom can be picked.
                                 Don't set this too high, since the chance
                                 for an exitroom to be picked from a pool of,
                                 say, 70 rooms is relatively slim already.
                                 10 might be a good choice.
    
  Props on each maze room [OPTIONAL]:
    _maze/exitroom:[dbref]   -  Set this prop on every maze room which allows 
                                leaving the maze. The prop must contain the
                                dbref of the action leading 'out' of the maze.
 
    _maze/trapdamage:[int]   -  If an integer value is set here, the HP of the
                                fur entering the room will be decreased by that
                                value. Yes, negative values 'heal' the player.
 
    _maze/chance:[int]       -  Specify the chance of the room to be chosen.
                                See _maze/defaultchance for an explanation.
                                This prop overrides defaultchance for just the
                                single room on which it is set.
 
  Props on actions [REQUIRED]:
    _startaction:[bool]       - This prop needs to be set on the action leading
                                into the maze. Set it to 1, or yes, or whatever
                                else comes to mind. It resets the player's HP
                                and step properties.
 
    _isMazeExit?:[bool]       - Set this property on the action leading out of
                                the maze in an exitroom; Hint: You used it's 
                                dbref in _maze/exitroom before. 
                                Set it to 'yes' to tell the program to treat 
                                this action as the actual maze exit.
 
  _maze/startingroom:yes must be set on the starting room of the maze.
  That's about it.
)
$def ParentData "_maze/" me @ intostr strcat
 
: getParent ( [  -- d ]  -  Retrieve dbref of parent of room player is in )
  me @ location location
;
 
: getReward ( [  -- i ]  -  Get amount of pennies given to successful furs )
  getParent
  "_maze/reward"
  getpropstr atoi
;
 
: getTarget ( [  -- d ]  -  Retrieve _targetroom property )
  getParent
  "_maze/targetroom"
  getpropstr
  "" "#" subst atoi dbref
;
 
: incSteps ( [  --  ]  -  Increase step counter )
  getParent
  ParentData "/steps" strcat
  over over getpropstr atoi 1 + intostr 1 addprop
;
 
: overMinSteps? ( [  -- b ]  -  Enough rooms trespassed to allow exit? )
  getParent
  ParentData "/steps" strcat
  getpropstr atoi
 
  getParent
  "_maze/minsteps"
  getpropstr atoi
  >=
;
 
: getStartHP ( [  -- i ]  -  Retrieve initial HP )
  getParent
  "_maze/starthp"
  getpropstr atoi
;
 
: getPlayerHP ( [  -- i ]  -  Retrieve HP of player )
  getParent
  ParentData "/hp" strcat
  getpropstr atoi
;
 
: setPlayerHP ( [ i --  ]  -  Set HP of player )
  getParent
  ParentData "/hp" strcat
  3 rotate intostr 0 addprop
;
 
: getChance ( [ d -- i ]  -  Get percentual chance of room d )
  "_maze/chance" getpropstr dup not 
  if
    pop getParent
    "_maze/defaultchance"
    getpropstr
  then
  atoi
  dup 100 > if pop 100 then
  dup 0 < if pop 0 then
;
 
: getExitAction ( [ d -- d' ]  -  If room d = exitroom return db# of exit )
  "_maze/exitroom"
  getpropstr dup if
    atoi dbref
  then
;
 
: isValidExit? ( [ d -- b ]  -  Check if d is a valid maze exit action )
  dup owner getParent owner dbcmp
  swap "_isMazeExit?" getpropstr and
;
 
: getTrapDamage ( [  -- i ]  -  Retrieve amount of damage room causes )
  me @ location
  "_maze/trapdamage"
  getpropstr atoi
;
 
: doInit ( [  --  ]  -  Reset a player's HP and steps properties )
  getStartHP setPlayerHP
  
  getParent
  ParentData "/steps" strcat
  remove_prop
;
 
: AssembleList ( [  --  ]  -  Build list of rooms. Needs a moment. )
  "Assembling Room List, please wait..." .tell
  getParent
  "_maze/roomlist" 
  remove_prop
 
  getParent contents
  begin
    dup #-1 dbcmp not while
    dup room?
    over owner getParent owner dbcmp and 
    over "_maze/startingroom" getpropstr not and if
      getParent
      "_maze/roomlist/" 
      over "_maze/roomlist/count" getpropstr atoi 1 + intostr strcat
      3 pick intostr 0 addprop
      getParent "_maze/roomlist/count"
      over over
      getpropstr atoi 1 + intostr
      0 addprop
    then
    next
  repeat
  pop
  
  getParent
  "_maze/roomlist/count"
  getpropstr atoi not if
    "Parentroom does not contain any usable maze rooms!" abort
  then
  "Done." .tell
;
 
: getRoomCount ( [  -- i ]  -  Retrieves number of rooms in maze pool )
  getParent
  "_maze/roomlist/count"
  getpropstr atoi
  dup not if 
    pop AssembleList getRoomCount
  then
;
 
: PickRoom ( [  -- d ]  -  Picks a room from the pool )
  "LOOP DUMMY"
  begin
   pop
   getParent
   random getRoomCount % 1 + intostr
   "_maze/roomlist/" swap strcat
   getpropstr atoi dbref      ( Room picked, now run tests.. )
 
   dup getExitAction not overMinSteps? or    ( Good ol' Boolean logic.. )
   over getChance random 100 % > and
   
  until 
;
 
: KillPlayer ( [  --  ]  -  Move player back home )
  " " .tell
  "The traps were too much for you.. you're sent home as your body can't" .tell
  "stand any more abuse." .tell
  " " .tell
  3 sleep
  me @ dup getlink moveto
;
 
: doMove ( [ d --  ]  -  Move Player to picked room d )
  me @ swap moveto
  incSteps
  getPlayerHP getTrapDamage -
  setPlayerHP
 
  getPlayerHP 0 <= if KillPlayer then
;
 
: LeaveMaze ( [  --  ]  -  Reward player and move him out of the maze )
  getParent
  ParentData 
  remove_prop
 
  "max_pennies" sysparm atoi me @ pennies -
  dup getReward < not if
    pop getReward 
  then
 
  dup 0 < if pop 0 then
 
  me @ over addpennies
  "You are given " swap intostr strcat 
  " " strcat "pennies" sysparm strcat
  " for completing the maze!" strcat .tell
  me @ getTarget moveto
;
 
: main
  pop
 
  trig isValidExit? if LeaveMaze exit then 
 
  trig "_startaction" getpropstr if doInit then
 
  PickRoom
  doMove
;

