( cmd-cp-mv2.muf  written by Foxx_Fox              93 FEB 12 )
(     updated version 2.1 by Foxx_Fox              93 NOV 15 )
( Great credit is given to Foxen's original cmd-cp-mv code   )
( Code for: cp; mv; lcp; lmv 
Interactive usage:
  
 *  cp [origobj][=prop],[destobj][=destprop]
      Copies prop from origobj to destobj, renaming it to destprop.
  
 *  mv [origobj][=prop],[destobj][=destprop]
      Moves prop from origobj to destobj, renaming it to destprop.
  
    lcp [origobj][=list],[destobj][=destlist]
      Copies prop from origobj to destobj, renaming it to destprop.
  
    lmv [origobj][=list],[destobj][=destlist]
      Moves prop from origobj to destobj, renaming it to destprop.
  
      Arguments in []s are optional.
      if origobj is omitted, it assumes a property on the user.
      if destobj is omitten, it assumes destobj is the same as origobj.
      if destprop is omitted, it assumes the same name as prop.
      if prop is omitted, it prompts for it.
      if both prop and origobj are omitted, it prompts for both.
      if both destobj and destprop are omitted, it prompts for them.
  
 * For these commands, 
    NOTE1: 'virtual props' such as @desc, @succ, can be copied/moved. 
    NOTE2: 'prop' may be a unix-style pattern such as 'com-*'
  
Public Routines callable by MUF programmers:
  
cp-prop [ d1 s1 d2 s2 -- count ]
  Copies prop/propdir 's1' from item 'd1' to item 'd2' renaming it to 's2'.
  'count' is an interger specifying the number of properties copied.
  
mv-prop [ d1 s1 d2 s2 -- count ]
  Moves prop/propdir 's1' from item 'd1' to item 'd2' renaming it to 's2'.
  'count' is an interger specifying the number of properties moved.
--- Notes about cp-prop and mv-prop
  These routines can copy/move entire propdirs and a unix-style pattern can
  be used in 's1'.  If you're only copying/moving one property you might want
  to consider using 'cp-1' or 'mv-1' instead.
  
lcp-prop [ d1 s1 d2 s2 -- count ]
  Copies a list named 's1' from item 'd1' to item 'd2' renaming it to 's2'.
  'count' is an interger specifying the number of properties copied.
  
lmv-prop [ d1 s1 d2 s2 -- count ]
  Moves a list named 's1' from item 'd1' to item 'd2' renaming it to 's2'.
  'count' is an interger specifying the number of properties moved.
  
--- Notes for lcp-prop and lmv-prop
  This list format simply starts placing 1, 2, 3... after the value of s1 and
  s2 and performs a series of copying/moving until no property is found.
  It will also try to put a '#' after the value for s1.
  [e.g. s1 = "list" will try to copy/move; list1, list2, ... list# ]
  
cp-1 ; mv-1 [ d1 s1 d2 s2 -- b ]
  Copies/moves a single property 's1' on 'd1' to 'd2' renaming it to 's2'.
  'b' is 1 if successful, 0 if not.  
  
confirm-cpmv [ d1 s1 d2 s2 -- d1 s1 d2 s2 b ]
  Confirms if d1 and d2 exist and cleans up slashes in s1 and s2.  Prints
  an error message if either object doesn't exist.  'b' = 1 if okay, 0 if not.
  
--- Notes for cp-1, mv-1 and confirm-cpmv
  These routines have no error checking and are designed specifically for a 
  single property, much faster than 'cp-prop' or 'mv-prop'.  If 'mv-1' is used 
  on a propdir will not erase any properties in that propdir.
  confirm-cpmv will take s1 and s2 and will strip any redundant slashes, and
  add a leading slash [blank values will be turned to "/"]
  Use 'confirm-cpmv' immediatly before 'cp-1' or 'mv-1' as a quick check like
  this. { e.g.  confirm-cpmv  if cp-1 then }
)  
  
( UPDATE NOTES
--- New Features:
  * @lock and the new @idescription are now supported by this program.
  * /<dir> to /<dir>/<subdir> transfer doesn't cause infinite loop bug.
  
--- Modified internal stuff:  
  * Copying and moving is now done by copying/moving all properties to a
    temporary property/propdir located on the user called 'cpmv<systime>'
    where <systime> is a string version of the current systime.
    The 'cpmv<systime>' prop is then copied to the appropriate location
    then erased.
  * All property names are set up to begin with '/'.  Some routines have
    been altered to insure this, and others altered to assume this.
  * New FB-MUF 5.x primatives have replaced $lib routines when possible.
  * getprop and setprop now used to transfer properties
) 
  
$include $lib/strings  ( .blank? ; .split )
$include $lib/match    ( .noisy_match )
$define DOHELP 
  trigger @ #1079 call  ( call help pager program )
$enddef
  
( Variables )
  
lvar copy  lvar silent  lvar pmatch  ( mode flags )
lvar t-flag ( mode flag use for temporary storage )
lvar count    ( i : Number of props copied/moved )
lvar pattern  ( s : pattern to search for for pattern matching )
lvar temp     ( s : temporary file used during transfer )
lvar d1  lvar d2  ( d : dbref of source and destination )
lvar s1  lvar s2  ( s : source and destination prop names )
  
( Variable handlers )
  
: copy?   copy @ ;
: silent? silent @ ;
: pmatch? pmatch @ ;
: store_ds_vars s2 ! d2 ! s1 ! d1 ! ;
: set_silent_move 1 silent ! 0 copy ! 0 count ! ;
: set_silent_copy 1 silent ! 1 copy ! 0 count ! ;
  
( Special string handling routines )
  
: tell ( s -- )
  ( notify user if not silent )
  silent? if
    pop
  else
    me @ swap notify
  then
;
  
: e-tell ( x x x x s -- )
  ( tell error string and clear stack )
  tell pop pop pop pop
;
  
: root? ( s -- b )
  ( gives 1 if 's' is root directory [ "" or "/" ] )
  dup not if pop 1 exit then
  "/" strcmp not
;
  
: process_vprop ( s -- s' )
  ( convert @desc, @succ, etc.. to property equivilant )
  dup "/@" 2 strncmp not if
    dup "/@de" 4 strncmp not if  pop "/_/de"  else  ( @describe )
    dup "/@s" 3 strncmp not  if  pop "/_/sc"  else  ( @success )
    dup "/@f" 3 strncmp not  if  pop "/_/fl"  else  ( @fail )
    dup "/@dr" 4 strncmp not if  pop "/_/dr"  else  ( @drop )
    dup "/@os" 4 strncmp not if  pop "/_/osc" else  ( @osuccess )
    dup "/@of" 4 strncmp not if  pop "/_/ofl" else  ( @ofail )
    dup "/@od" 4 strncmp not if  pop "/_/odr" else  ( @odrop )
    dup "/@i" 3 strncmp not  if  pop "/_/ide" else  ( @idescribe )
    dup "/@l" 3 strncmp not  if  pop "/_/lok" else  ( @lock )
    dup "/@a" 3 strncmp not  if  pop "/_"           ( @all )
    then then then then  then then then then  then then
  then 
;
  
: set_esc_chars ( s -- s' ) ( hide special chars if preceeded by '\' )
  "AST" "\\*"  subst  "QST" "\\?"  subst
  "OBK" "\\["  subst  "CBK" "\\]"  subst
  "OBR" "\\}"  subst  "CBR" "\\}"  subst
  "BSL" "\\\\" subst  "" "\\" subst 
;
  
: unset_esc_del ( s' -- s ) ( undo set_esc_chars, delete '\'s )
  "*" "AST"  subst  "?" "QST"  subst
  "[" "OBK"  subst  "]" "CBK"  subst
  "{" "OBR"  subst  "}" "CBR"  subst
  "\\" "BSL" subst
;
  
: unset_esc_keep ( s' -- s ) ( undo set_esc_chars, keeping '\'s )
  "\\*" "AST" subst  "\\?" "QST"  subst
  "\\[" "OBK" subst  "\\]" "CBK"  subst
  "\\{" "OBR" subst  "\\}" "CBR"  subst
  "\\"  "BSL" subst
;
  
: strip-slashes ( s -- s' )
  begin
    dup "/" "//" subst    ( s s1 )
  over over strcmp while  ( exit loop if equal )
    swap pop              ( s1 )
  repeat pop
;
  
: set-slashes ( s -- s' )
  ( delete doubled-up/trailing and add leading '/'s )
  ( strips tail if t-flag is set... t-flag values: 1=cp/mv 0=lcp/lmv )
  dup root? if pop "/" exit then   ( "/" or "" cases )
  "/" swap strcat                  ( add "/" )
  strip-slashes
  dup strlen        ( s len )
  dup 1 = if pop exit then  ( "//" case )
  t-flag @ if
    1 - strcut
    dup "/" strcmp if strcat else pop then ( trailing slash )
  else pop
  then 
;
  
: smatch+ ( str patt -- b ) 
  ( patch to fix 'bugs' with standard smatch )
  over .blank? over .blank? or if  
    pop pop 0 exit  
  then
  smatch
;
  
: finale ( n -- )  ( print 'done' and total number of props copied/moved )
  silent? if
    pop
  else
    ">> Done.  <n> propert<y|ies> <verb>."
    copy?     if "copied" else "moved" then    "<verb>" subst
    over 1 =  if "y" else "ies" then          "<y|ies>" subst
    swap intostr                            "<n>" subst .tell
  then
;
  
( Preprocessors )
  
: confirm ( d1 s1 d2 s2 -- d1 s1 d2 s2 b ) 
  ( needs: t-flag = 1 for cp/mv, 0 for lcp/lmv )
  ( confirm d1 & d2 are ok, s1 & s2 are striped)
  over ok? not   if ">> Bad source object." e-tell 0 exit then
  4 pick ok? not if ">> Bad destination object." e-tell 0 exit then
  over 5 pick dbcmp over 5 pick strcmp not and ( d1=d2 & s1=s2 )
    if ">> Source and destination identical." e-tell 0 exit then
  tolower strip set-slashes 
  rot tolower strip set-slashes rot rot
  1
;
  
: control_bad? ( -- b )
  ( returns 1 if user doesn't control the object )
  me @ d1 @ controls not if
    "You don't control source object." tell
    1 exit 
  then
  me @ d2 @ controls not if
    "You don't control destination object." tell
    1 exit
  then
  0
;
  
: preprocess ( -- )
  ( handle v-props, set smatch flag, adjust s1 and s2 if needed )
  s1 @ set_esc_chars       ( s1+ )
  dup "{*\\**|*\\?*|*\\[*\\]*|*\\{*\\}*}" smatch+ if 
    ( pattern match; s1 end in / )
    dup unset_esc_keep pattern !  ( s1+ )
    1 pmatch !                    ( s1+ )
    unset_esc_del                 ( s1 )
    begin 
      dup "/" rinstr                ( s1 i )
      dup 1 > if ( clip to last propdir )
        over strlen over = if  ( last char = "/" ? )
          1 - strcut pop              ( s1/ss1-/ )
          dup "/" rinstr              ( s1/ss1-/ i )
        then  
        strcut pop                  ( s1-/ )
      else
        pop pop "/"                 ( / )
      then
      dup "{*\\**|*\\?*|*\\[*\\]*|*\\{*\\}*}" smatch not
    until
    dup strlen 1 - strcut pop ( drop last / )           
      ( s1- )
    s1 @ s2 @ strcmp not if ( if equal, reset s2 )
      dup s2 !
    then
    s1 !   
  else ( no pattern match )
    0 pmatch !
    unset_esc_del process_vprop s1 !
    s2 @ process_vprop s2 !
  then
;  
  
( Property handling routines )
  
: remove_prop+ ( d1 s1 -- )
  ( removes property without erasing entire propdir )
  over over propdir? if
    "" 0 addprop
  else
    remove_prop
  then 
;
  
( Copy/Move property routines )
  
: cp-mv-1 ( d1 s1 d2 s2 -- d1 s1 d2 s2 b )
  ( copy or move 1 property. b=1 if done )
  dup root? if 0 exit then
  over over 6 pick 6 pick getprop ( d1 s1 d2 s2 d2 s2 d1:s1 )
  dup not if  ( prop doesn't exist )
    pop pop pop 0                 ( d1 s1 d2 s2 0 )
  else
    setprop 1                     ( d1 s1 d2 s2 1 )
  then
  
  dup if ( update counter and cleanup )
    count @ 1 + count !
    copy? not if ( move )
      5 pick 5 pick               ( d1 s1 d2 s2 b d1 s2 )
      remove_prop+                ( d1 s1 d2 s2 b )
    then    
  then
;
  
( -- Propdir cp/mv cycle -- )
  
: cp-mv-cycle ( d1 s1 d2 s2 -- )
  ( Cycle through props and copy or move them )
  ( handle single property )
  3 pick root? if
    1                       ( d1 s1 d2 s2 1 )
  else
    cp-mv-1                 ( d1 s1 d2 s2 b2 )
    5 pick 5 pick propdir?  ( d1 s1 d2 s2 b2 b1 )
    swap over or            ( d1 s1 d2 s2 b1 b1|b2 )
    not if ( neither propdir nor ok )
      silent? not if
        "I don't see what property to "
        copy? if "copy." else "move." then strcat
        e-tell exit
      then
    then
  then  ( d1 s1 d2 s2 b1 )
  
  if ( handle propdir )
    4 pick dup 5 pick "/" strcat nextprop ( d1 s1 d2 s2 d1 s1/ss1 )
    begin
    dup while ( while next prop exists )
      over dup 3 pick nextprop  ( .. d2 s2 d1 s1/ss1 d1 s1/ss1' )
      4 rotate 4 rotate         ( .. d2 s2 d1 s1/ss1' d1 s1/ss1 )
      6 pick 6 pick "/" strcat  ( .. d2 s2 d1 s1/ss1' d1 s1/ss1 d2 s2/ )
      3 pick dup "/" rinstr     ( .... d1 s1/ss1' d1 s1/ss1 d2 s2/ s1/ss1 i )
      strcut swap pop strcat    ( .... d1 s1/ss1' d1 s1/ss1 d2 s2/ss1 )
      cp-mv-cycle               ( .... d1 s1/ss1' )
    repeat ( stack set for next iteration )
    pop pop ( d1 s1 d2 s2 )
  then
  pop pop pop pop 
;
  
( -- Pattern matching cp/mv cycle )
  
: cp-mv-smatch ( d1 s1 d2 s2 -- )
  ( Cycle through props and copy/move only those matching pattern )
  ( handle single property )
  3 pick root? if
    1                       ( d1 s1 d2 s2 1 )
  else
    3 pick pattern @ smatch+ if
      cp-mv-1 pop           ( d1 s1 d2 s2 )
    then
    4 pick 4 pick propdir? if
      3 pick "/" strcat pattern @ "/" strcat  ( .... s1/ patt/ )
      begin
        dup root?   if pop pop 0 break then ( no match, ingore propdir )
        over over smatch+  if pop pop 1 break then ( match, look in propdir )
        dup dup strlen 1 - strcut pop         ( .... s1/ patt/ patt )
        "/" rinstr strcut pop                 ( .... s1/ patt-/ )
      repeat
    else  0  ( not a propdir )
    then 
  then
  
  if ( handle propdir )
    4 pick dup 5 pick "/" strcat nextprop ( d1 s1 d2 s2 d1 s1/ss1 )
    begin
    dup while ( while next prop exists )
      over dup 3 pick nextprop  ( .. d2 s2 d1 s1/ss1 d1 s1/ss1' )
      4 rotate 4 rotate         ( .. d2 s2 d1 s1/ss1' d1 s1/ss1 )
      6 pick 6 pick "/" strcat  ( .. d2 s2 d1 s1/ss1' d1 s1/ss1 d2 s2/ )
      3 pick dup "/" rinstr     ( .... d1 s1/ss1' d1 s1/ss1 d2 s2/ s1/ss1 i )
      strcut swap pop strcat    ( .... d1 s1/ss1' d1 s1/ss1 d2 s2/ss1 )
      cp-mv-smatch              ( .... d1 s1/ss1' )
    repeat ( stack set for next iteration )
    pop pop ( d1 s1 d2 s2 )
  then
  pop pop pop pop 
;
  
( List cp-mv )
  
: lcp-lmv-prop ( d1 s1 d2 s2 - count )
  d1 @ s1 @ d2 @ s2 @ 0 count ! 0
  begin
    1 +
    5 pick 5 pick 3 pick intostr strcat   ( .... i d1 s1i )
    5 pick 5 pick 5 pick intostr strcat   ( .... i d1 s1i d2 s2i )
    cp-mv-1 -5 rotate pop pop pop pop     ( .... i b )
  not until
  pop   ( d1 s1 d2 s2 )
  rot "#" strcat -3 rotate
  "#" strcat cp-mv-1 pop
  pop pop pop pop
  count @
;
  
( Normal cp-mv )
  
: cp-mv-prop ( -- count )
  begin
    "/cpmv" systime intostr strcat dup temp !  ( temp ) ( store temporary )
    prog over getprop not not ( temp prog:temp )
    prog rot propdir?         ( prop:temp propdir )
  or while  ( temporary exists )
    1 sleep  ( wait a second until temp goes away due to another program )
  repeat
  
  d1 @ s1 @ prog temp @ 0 count !
  pmatch? if
    cp-mv-smatch
  else
    cp-mv-cycle
  then
  count @ if  ( if any matches, then transfer to destination using cp )  
    copy @ t-flag ! 1 copy !
    prog temp @ d2 @ s2 @ 0 count !
    cp-mv-cycle
    t-flag @ copy !
  then
  prog temp @ remove_prop
  count @
;
  
  
( MUF accessable public routines )
  
( t-flag = 0 for list, 1 for no list... used in 'confirm' )
  
: mv-prop ( d1 s1 d2 s2 -- count )
  set_silent_move 1 t-flag ! 
  confirm not if pop pop pop pop 0 exit then
  store_ds_vars
  preprocess 
  cp-mv-prop
;
  
: cp-prop ( d1 s1 d2 s2 -- count )
  set_silent_copy 1 t-flag ! 
  confirm not if pop pop pop pop 0 exit then
  store_ds_vars
  preprocess
  cp-mv-prop
;
  
: lmv-prop ( d1 s1 d2 s2 -- count )
  set_silent_move 0 t-flag ! 
  confirm not if pop pop pop pop 0 exit then
  store_ds_vars
  lcp-lmv-prop
;
  
: lcp-prop ( d1 s1 d2 s2 -- count )
  set_silent_copy 0 t-flag ! 
  confirm not if pop pop pop pop 0 exit then
  store_ds_vars
  lcp-lmv-prop
;
  
: cp-1 ( d1 s1 d2 s2 -- b )
  dup root? if pop pop pop pop 0 exit then
  getprop
  dup not if
    pop pop pop 0 exit
  else
    setprop 1
  then
;
  
: mv-1 ( d1 s1 d2 s2 -- b )
  4 pick 4 pick 4 rotate 4 rotate  ( d1 s1 d1 s1 d2 s2 )
  cp-1    ( d1 s1 b )
  dup if
    rot rot  ( 1 d1 s1 )
    remove_prop+
  else
    swap pop swap pop  ( 0 )
  then
;
  
: confirm-cpmv ( d1 s1 d2 s2 -- b )
  1 t-flag ! 0 silent !
  confirm
; 
  
( Parser and Main Routines )
  
: parse_input ( s -- b ) ( b=1 ok )
  ( assume command in the form <src_obj><=src_prop>,<desc_obj><=desc_prop> )
  
  ( work with source object and property )
  "," .split strip swap strip    ( "d2=p2" "d1=p1" )
  "=" .split strip swap strip    ( "d2=p2" "p1" "d1" )
  
  dup if  ( d1 given )
    .noisy_match                   ( "d2=p2" "p1" d1 )
    dup not if pop pop pop 0 exit then  ( null match.. exit program )
  else  ( d1 !given )
    over not if ( p1 !given )
      "Please enter name of original object" tell
       pop read strip             ( "d2=p2" "p1" "d1" )
    then
    dup if  ( d1 given )
      .noisy_match                 ( "d2=p2" "p1" d1 )
      dup not if pop pop pop 0 exit then  ( null match.. exit program )
    else ( d1 still !given , assume d1=me )
      pop me @
    then
  then
  swap                             ( "d2=p2" d1 "p1" )
  begin ( loop to get non-blank s1 after strip & strip-slashes )
    strip-slashes
    dup not while ( s1 exists? .. exit loop )
    "Please enter the name of the original property." tell
    pop read strip
  repeat                           ( "d2=p2" d1 s1 )
  set-slashes         ( final cleanup for s1 )
  s1 ! d1 !                        ( "d2=p2" )
  
  ( work with destination object and property )
  "=" .split strip strip-slashes  ( "d2" s2 )
  swap strip                      ( s2 "d2" )
  begin
    over over or not while ( s2 or d2 exists? .. exit loop )
    pop pop
    "Please enter the name of the destination object." tell
    read strip
    "Please enter the name of the destination property." tell
    read strip
    strip-slashes swap
  repeat                           ( s2 "d2" )
  dup if ( d2 string exists )
    .noisy_match
    dup not if pop pop 0 exit then ( null match .. exit program )
  else ( d2 !exist?, use d1 )
    pop d1 @
  then                             ( s2 d2 )
  d2 !                             ( s2 )
  dup not if ( s2 !exists?, use s1 )
    pop s1 @
  else
    set-slashes
  then
  s2 !                             ( )
  d1 @ d2 @ dbcmp s1 @ s2 @ strcmp not and if
    "Source and destination, props and objects identical." tell
    0 exit
  then 
  1                                ( 1 )
;
  
: do_help
  DOHELP
;
  
: main ( s -- )
  0 silent !
  command @ tolower "c" instr not not copy !
  
  dup tolower "#h" 2 strncmp not if
    pop
    do_help
    exit
  then
  
  command @ tolower "l" instr not t-flag ! ( t-flag = !list )
  parse_input not if ">> End program" tell exit then
  control_bad? if exit then
  t-flag @ if
    preprocess
    cp-mv-prop
  else
    lcp-lmv-prop
  then
  finale  
;
  
public mv-prop
public cp-prop
public lmv-prop
public lcp-prop
public mv-1
public cp-1
public confirm-cpmv

