( spar-roster.muf     by Gyroplast  02/02/2001   v1.03
  
  This prog contains all necessary functions for the list administration of
  spar.muf. Linked to the enroll;lookup command.
 
  Exports:
  enrolled?      [ d -- b ]
    Is the referenced fur enrolled already?
 
  getdbobj       [ v -- i ]
    Put database object# in v, i contains errorlevel;
      0 - No error
      1 - .database prop not set on action or invalid
      2 - .database is not a thing, or not owned by action owner
 
  gethp/sethp          [ d -- i ] / [ d i --  ]
    Get/Set current health points
 
  getmaxhp/setmaxhp    [ d -- i ] / [ d i --  ]
    Get/Set maximum health points
 
  getdex/setdex        [ d -- i ] / [ d i --  ]
    Get/Set current dexterity
 
  getmaxdex/setmaxdex  [ d -- i ] / [ d i --  ]
    Get/Set maximum dexterity
 
  getstr/setstr        [ d -- i ] / [ d i --  ]
    Get/Set current strength
 
  getmaxstr/setmaxstr  [ d -- i ] / [ d i --  ]
    Get/Set maximum strength
 
  getexp/setexp        [ d -- i ] / [ d i --  ]
    Get/Set experience
 
  getlevel/setlevel    [ d -- i ] / [ d i --  ]
    Get/Set numeric level code of player d
  level2rank     [ i -- s ]
    Convert numeric level to a more legible rank string
  getwonmatches/setwonmatches  [ d -- i ] / [ d i --  ]
    Get/Set number of player d's won matches
  getrank        [ d -- s ]
    Shortcut for 'getlevel level2rank'. Retrieves the rank string of player d
  If you're installing the program the first time, register it as 
  spar/spar-roster with this wiz command:
   @reg #prop #0:_reg spar-roster.muf=spar/spar-roster
 
  If you're not a wiz, exchange the #0 to the dbref of your environment room.
 
)
$include $lib/reflist
$include $lib/strings
 
$def header "Spar-Roster.muf [v1.03] by Gyroplast@FuzzyLogic (01/31/2000)"
$def pad STRleft
$def center 79 STRcenter
 
lvar count
var db
 
( --- General functions --- )
 
: getdbobj ( v -- i )( put db# of database into v, i = errorlevel )
  trig ".database" envpropstr swap pop atoi dup not if
   pop pop 
   "No database specified. Contact " trig owner name strcat
   "." strcat .tell 1 exit
  then
  dbref dup ok? not if
   pop pop
   "The specified database object does not exist. Contact "
   trig owner name strcat "." strcat .tell 2 exit
  then
  dup owner trig owner dbcmp not over thing? not or if
   pop pop
   "The specified database either is not a valid THING, or not owned by "
   trig owner name strcat "." strcat .tell
   "Please contact " trig owner name strcat "." strcat .tell 2 exit
  then
  swap ! 0
;
 
: enrolled? ( d -- b )( check if player is enrolled )
  db @ swap "_list/enlisted" swap REF-inlist?
;
 
: gethp ( d -- i )( get current health points )
  db @ swap "_list/" swap intostr strcat "/hp" strcat getpropstr atoi
;
  
: sethp ( d i -- )( set current health points to i )
  "_list/" rot intostr strcat "/hp" strcat
  db @ swap rot intostr 0 addprop
;
: getmaxhp ( d -- i )( get maximum health points )
  db @ swap "_list/" swap intostr strcat "/maxhp" strcat getpropstr atoi
;
: setmaxhp ( d i -- )( set maximum health points to i )
  "_list/" rot intostr strcat "/maxhp" strcat
  db @ swap rot intostr 0 addprop
;
 
: getstr ( d -- i )( get current strength )
  db @ swap "_list/" swap intostr strcat "/str" strcat getpropstr atoi
;
: setstr ( d i -- )( set current strength to i )
  "_list/" rot intostr strcat "/str" strcat
  db @ swap rot intostr 0 addprop
;
 
: getmaxstr ( d -- i )( get maximum strength )
  db @ swap "_list/" swap intostr strcat "/maxstr" strcat getpropstr atoi
;
 
: setmaxstr ( d i -- )( set maximum strength to i )
  "_list/" rot intostr strcat "/maxstr" strcat
  db @ swap rot intostr 0 addprop
;
: getdex ( d -- i )( get current dexterity )
  db @ swap "_list/" swap intostr strcat "/dex" strcat getpropstr atoi
;
  
: setdex ( d i -- )( set current dexterity to i )
  "_list/" rot intostr strcat "/dex" strcat
  db @ swap rot intostr 0 addprop
;
: getmaxdex ( d -- i )( get maximum dexterity )
  db @ swap "_list/" swap intostr strcat "/maxdex" strcat getpropstr atoi
;
 
: setmaxdex ( d i -- )( set maximum dexterity to i )
  "_list/" rot intostr strcat "/maxdex" strcat
  db @ swap rot intostr 0 addprop
;
: getexp ( d -- i )( get experience )
  db @ swap "_list/" swap intostr strcat "/exp" strcat getpropstr atoi
;
 
: setexp ( d i -- )( set experience to i )
  "_list/" rot intostr strcat "/exp" strcat
  db @ swap rot intostr 0 addprop
;
: getlevel ( d -- i )( get numeric level of fighter )
  db @ swap "_list/" swap intostr strcat "/level" strcat getpropstr atoi
;
 
: setlevel ( d i -- )( set numeric level of fighter to i )
  "_list/" rot intostr strcat "/level" strcat
  db @ swap rot intostr 0 addprop
;
: getwonmatches ( d -- i )( get number of won matches )
  db @ swap "_list/" swap intostr strcat "/won" strcat getpropstr atoi
;
 
: setwonmatches ( d i -- )( set number of won matches to i )
  "_list/" rot intostr strcat "/won" strcat
  db @ swap rot intostr 0 addprop
;
: level2rank ( i -- s )( convert level to rank string )
  intostr db @ swap "_ranks#/" swap strcat getpropstr
  dup not if "No rank set for this level. Contact " prog owner name strcat
  "!" strcat .tell then
;
 
: getrank ( d -- s )( functionalized macro to retrieve rank string from d )
  getlevel level2rank
;
 
( --- enroll function --- )
: enroll (  --  )( add a fur to the list )
  me @ enrolled? if 
    "You are already enlisted." .tell 
    "Continuing will RESET all your stats permanently!" .tell
    "Are you absolutely sure you want to continue? [y/N]" .tell
    READ strip
    "y" stringpfx 1 = not if "Aborted." .tell exit then
  then
  db @ "_list/enlisted" me @ REF-add
  ( SET INITIAL VALUES )
  me @
  dup 1 setlevel
  dup 10 sethp
  dup 10 setmaxhp
  dup 10 setstr
  dup 10 setmaxstr
  dup 10 setdex
  dup 10 setmaxdex
  dup 10 setexp
  0 setwonmatches
  "Congratulations, you are now an enlisted member." .tell
  "Feel free to roam around, train hard, and spar a bit!" .tell
;
 
( --- lookup functions --- )
( general lookup function for listall and listhere)
: namelookup ( d -- s )( retrieve name and rank )
  dup getrank 55 STRright
  over name strlen strcut swap pop
  swap name "           " swap strcat swap strcat
;
 
( LOOKUP <name> )
: showinfo ( d --  )( show name and rank of <name> )
  " " .tell
  "====---------------====---------------====" center .tell
  dup name 42 STRcenter
  3 strcut swap pop " ][" swap strcat
  dup strlen 3 - strcut pop "][ " strcat
  center .tell
  "====---------------====---------------====" center .tell
  dup getrank center .tell
  "====---------------====---------------====" center .tell
  dup getwonmatches intostr " matches won." strcat 42 STRcenter
  3 strcut swap pop " ][" swap strcat
  dup strlen 3 - strcut pop "][ " strcat
  center .tell
  "====---------------====---------------====" center .tell
  " " .tell
;
 
( LOOKUP #all )
: listall (  --  )( lists all enlisted furs )
  ",-----------------------------------------------------------------------------." .tell
  "| List of all enrolled furs                                                   |" .tell
  "`-----------------------------------------------------------------------------'" .tell
  "'                                                                             `" .tell
  1 count !
  db @ "_list/enlisted" REF-first
  BEGIN
   dup #-1 dbcmp not while
   dup namelookup .tell
   count @ 1 + dup count ! 20 > if
    "Please enter 'q' to stop, or anything else to continue." .tell
    read strip
    "q" stringcmp not if break then
   then
  db @ "_list/enlisted" 3 rotate REF-next
  REPEAT
  pop
  ".                                                                             ," .tell
  ",-----------------------------------------------------------------------------." .tell
  "|                                                                             |" .tell
  "`-----------------------------------------------------------------------------'" .tell
;
 
( LOOKUP #here )
: listhere (  --  )( shows all enlisted furs in the current room )
  ",-----------------------------------------------------------------------------." .tell
  "| Enrolled furs in the current room                                           |" .tell
  "`-----------------------------------------------------------------------------'" .tell
  "'                                                                             `" .tell
  1 count !
  me @ location contents
  BEGIN
   dup #-1 dbcmp not while
   dup player? not over enrolled? not or if next continue then
   dup namelookup .tell
   count @ 1 + dup count ! 20 > if
    "Please enter 'q' to stop, or anything else to continue." .tell
    read strip
    "q" stringcmp not if break then
   then
   next
  REPEAT
  pop
  ".                                                                             ," .tell
  ",-----------------------------------------------------------------------------." .tell
  "|                                                                             |" .tell
  "`-----------------------------------------------------------------------------'" .tell
;
 
( LOOKUP #ranks )
: listranks (  --  )( lists all available ranks )
  ",----------------------------------------------------------------." center .tell
  "|  " header strcat "  |" strcat center .tell
  "`================================================================'" center .tell
  "'                     -----=[ RANKS ]=-----                      `" center .tell
  " " .tell
  1 count !
  1
  BEGIN
   db @ "_ranks#/" 3 pick intostr strcat getpropstr dup while
   center .tell
   count @ 1 + dup count ! 20 > if
    "Please enter 'q' to stop, or anything else to continue." .tell
    read strip
    "q" stringcmp not if break then
   then
   1 +
  REPEAT
  pop pop
  ".                                                                ," center .tell
  ",----------------------------------------------------------------." center .tell
  "|                                                                |" center .tell
  "`----------------------------------------------------------------'" center .tell
;
 
( --- Help screen --- )
: showhelp (  --  )( show the help screen. Duh. )
  ",----------------------------------------------------------------." center .tell
  "|  " header strcat "  |" strcat center .tell
  "`================================================================'" center .tell
  "'                                                                `" center .tell
  "|  lookup <name>    :  Look up a player's rank" 65 pad "|" strcat center .tell
  "|  lookup #all      :  List players and their ranks" 65 pad "|" strcat center .tell
  "|  lookup #here     :  Look up ranks of those being here" 65 pad "|" strcat center .tell
  "|  lookup #ranks    :  List all available ranks" 65 pad "|" strcat center .tell
  "|  lookup #help     :  This helpful screen" 65 pad "|" strcat center .tell
  "|  " 65 pad "|" strcat center .tell
  "`--------------------------------=[ gyroplast@gyroplast.com ]=---'" center .tell
;
 
( --- MAIN FUNCTION / COMMAND PARSER --- )
: mane
  db getdbobj if exit then
  command @ "enroll" stringcmp not if pop enroll exit then
  strip
  dup not if pop showhelp exit then
  dup "#a" stringpfx 1 = if pop listall exit then
  dup "#hel" stringpfx 1 = if pop showhelp exit then
  dup "#her" stringpfx 1 = if pop listhere exit then
  dup "#r" stringpfx 1 = if pop listranks exit then
  dup "#" stringpfx 1 = if pop showhelp exit then
 
  .pmatch
  dup player? not if 
   "I am not aware of a player with that name." .tell exit
  else
   dup enrolled? not if
    pop "This player is not listed." .tell exit
   else
    showinfo exit
   then
  then
;
PUBLIC getdbobj
PUBLIC enrolled?
PUBLIC gethp
PUBLIC getmaxhp
PUBLIC getstr
PUBLIC getmaxstr
PUBLIC getdex
PUBLIC getmaxdex
PUBLIC getexp
PUBLIC getlevel
PUBLIC sethp
PUBLIC setmaxhp
PUBLIC setstr
PUBLIC setmaxstr
PUBLIC setdex
PUBLIC setmaxdex
PUBLIC setexp
PUBLIC setlevel
PUBLIC level2rank
PUBLIC getwonmatches
PUBLIC setwonmatches
PUBLIC getrank
$echo "ATTENTION! This program is in Beta phase, porting is EXPLICITLY NOT ALLOWED!"

